﻿//-----------------------------------------------------------------------------
// Copyright 2013 by VueSoft LLC. All rights reserved.
// http://www.printablecal.com
//-----------------------------------------------------------------------------
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of "PrintableCal" nor the names of its contributors may 
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Data;
using System.Data.OleDb;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Vuesoft.PrintableCal.Common;
using Vuesoft.PrintableCal.Interfaces;

namespace AccessDbCalendarSource
{
   /// <summary>
   /// This class provides a simple example of how PrintableCal can be extended to support additional data sources. In this
   /// example, the data source comes from an Access database. The same basic concepts would apply to any other type of data
   /// source. 
   /// <remarks>
   /// When defining a custom data source, make sure to add the Export and ExportMetadata attributes. These are required for
   /// PrintableCal to recognize the type is an extension. 
   /// </remarks>
   /// </summary>
   [Export(typeof(ICalendarSource))]
   [ExportMetadata("SourceName", AccessDbCalendarSource.SOURCE_NAME)]
   public class AccessDbCalendarSource : ICalendarSource
   {
      /// <summary>
      /// The source name. This name is used in the PrintableCal calendar list and when comparing calendar data sources to
      /// determine which ones are implemented by this class. The name must be unique!
      /// </summary>
      public const string SOURCE_NAME = "Example Access Database";

      /// <summary>
      /// The calendar name. It doesn't need to be unique. It comes from the CalendarName field in the Calendars table of the
      /// example database.
      /// </summary>
      private string m_calendarName;

      /// <summary>
      /// The database file location, including the full path. This value is used to build the connection string.
      /// </summary>
      private string m_dataLocation = "";

      /// <summary>
      /// The calendar ID. It must be unique. It comes from the CalendarID field of the Calendars table in the example database.
      /// </summary>
      private string m_calendarId = new Guid().ToString();

      /// <summary>
      /// The prefix to be added to items in generated calendars. This value is entered by the user when the calendar is selected
      /// in the calendar list.
      /// </summary>
      private string m_itemPrefix = "";

      /// <summary>
      /// A list of all schedule items associated with this calendar data source. This list is initialized by the 
      /// InitializeItems function, which gets commanded by PrintableCal before generating a calendar. The list is populated by
      /// querying the Events table of the example database.
      /// </summary>
      private List<IScheduleItem> m_items = new List<IScheduleItem>();

      /// <summary>
      /// The appearance to use for events associated with this calendar data source. The appearance will default to a random
      /// appearance. It can be modified by the user by clicking the calendar's name in the calendar list to access the calendar
      /// properties.
      /// </summary>
      private IAppearance m_itemAppearance = null;

      /// <summary>
      /// A flag indicating whether this calendar data source is selected for inclusion in generated calendars. A calendar data
      /// source is "selected" by the user checking the box next to the calendar name.
      /// </summary>
      private bool m_isSelected = true;

      /// <summary>
      /// Gets the icon associated with this calendar data source. The icon is shown in the main PrintableCal user interface, 
      /// when listing the calendar data sources that have been added and when displaying the drop-down menu after the user clicks
      /// the "Add" button.
      /// </summary>
      public System.Drawing.Image CalendarIcon
      {
         get { return Properties.Resources.icon; }
      }

      /// <summary>
      /// The calendar ID. It must be unique. It comes from the CalendarID field of the Calendars table in the example database.
      /// </summary>
      public string CalendarId
      {
         get { return m_calendarId;  }
         set { m_calendarId = value; }
      }

      /// <summary>
      /// The calendar name. It doesn't need to be unique. It comes from the CalendarName field in the Calendars table of the
      /// example database.
      /// </summary>
      public string CalendarName
      {
         get { return m_calendarName; }
         set { m_calendarName = value; }
      }

      /// <summary>
      /// Creates and returns a new instance of the calendar data source. This function is commanded by FormAddCalendar.
      /// </summary>
      /// <returns>Returns a new instance of AccessDbCalendarSource.</returns>
      public ICalendarSource Create()
      {
         return new AccessDbCalendarSource();
      }

      /// <summary>
      /// The database file location, including the full path. This value is used to build the connection string.
      /// </summary>
      public string DataLocation
      {
         get { return m_dataLocation; }
         set { m_dataLocation = value; }
      }

      /// <summary>
      /// This function initializes this calendar data source with a list of events occurring within the specified date range.
      /// This function is commanded by PrintableCal before a calendar is generated. The Events table of the example database 
      /// is queried to fill the event list. The CustomData field of the database is appended to event descriptions to provide
      /// an example of how additional custom fields could be generically integrated into the generated output.
      /// </summary>
      /// <param name="startRange">The start date.</param>
      /// <param name="endRange">The end date.</param>
      public void InitializeItems(DateTime startRange, DateTime endRange)
      {
         // clear any items if this is not the first time the function was commanded.
         this.Items.Clear();

         // use the data location to create a connection string
         string connectionString = @"Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + m_dataLocation;

         using (OleDbConnection connection = new OleDbConnection(connectionString))
         {
            connection.Open();

            // fill the dataset
            DataSet dataSet = new DataSet();

            using (OleDbDataAdapter dataAdapter = new OleDbDataAdapter("SELECT * FROM Events WHERE CalendarID='" + m_calendarId + "'", connection))
            {
               dataAdapter.Fill(dataSet, "Events");

               foreach (DataRow eventRow in dataSet.Tables["Events"].Rows)
               {
                  DateTime startDateTime = Convert.ToDateTime(eventRow["StartDateTime"]);
                  DateTime endDateTime = Convert.ToDateTime(eventRow["EndDateTime"]);
                  bool allDay = Convert.ToBoolean(eventRow["AllDay"]);

                  if (allDay)
                  {
                     // adjust the start and end values if the event is all-day. The start value must begin at the start of the
                     // day and the end value must fill the entire day, ending 1 second before midnight.
                     startDateTime = startDateTime.Date;
                     endDateTime = endDateTime.Date.AddSeconds(86399);
                  }

                  // check if the event start/end time intersects the specified date range - only add the event if it's within range
                  if (CalendarUtils.TimeRangesIntersect(startRange, endRange, startDateTime, endDateTime))
                  {
                     string eventId = eventRow["EventID"].ToString();
                     string title = eventRow["Title"].ToString();
                     string location = eventRow["Location"].ToString();
                     string description = eventRow["Description"].ToString();
                     string customData = eventRow["CustomData"].ToString();

                     IScheduleItem item = new AccessDbScheduleItem(this, title, description, startDateTime, endDateTime, allDay, location, customData);
                     Items.Add(item);
                  }
               }
            }

            // close the connection
            connection.Close();
         }
      }

      /// <summary>
      /// Always false, since the example database is stored locally.
      /// </summary>
      public bool IsOnline
      {
         get { return false; }
      }

      /// <summary>
      /// Get/sets a flag indicating whether this calendar data source is selected for inclusion in generated calendars. A calendar data
      /// source is "selected" by the user checking the box next to the calendar name.
      /// </summary>
      public bool IsSelected
      {
         get { return m_isSelected; }
         set { m_isSelected = value; }
      }

      /// <summary>
      /// Gets/sets the appearance to use for events associated with this calendar data source. The appearance will default to a random
      /// appearance. It can be modified by the user by clicking the calendar's name in the calendar list to access the calendar
      /// properties.
      /// </summary>
      public IAppearance ItemAppearance
      {
         get { return m_itemAppearance; }
         set { m_itemAppearance = value; }
      }

      /// <summary>
      /// Gets/sets the prefix to be added to items in generated calendars. This value is entered by the user when the calendar is selected
      /// in the calendar list.
      /// </summary>
      public string ItemPrefix
      {
         get { return m_itemPrefix; }
         set { m_itemPrefix = value; }
      }

      /// <summary>
      /// Gets a list of all schedule items associated with this calendar data source. This list is initialized by the 
      /// InitializeItems function, which gets commanded by PrintableCal before generating a calendar. The list is populated by
      /// querying the Events table of the example database.
      /// </summary>
      public List<IScheduleItem> Items
      {
         get { return m_items; }
      }

      /// <summary>
      /// This function is not required in this example. It could be used if there were additional settings required for 
      /// connecting to the data source, such as a username and password.
      /// </summary>
      /// <param name="settings">The settings collection</param>
      public void LoadSettings(Dictionary<string, object> settings)
      {
         // no custom settings are required for this data source
      }

      /// <summary>
      /// This function is not required in this example. It could be used if there were additional settings required for 
      /// connecting to the data source, such as a username and password.
      /// </summary>
      /// <param name="settings">The settings collection</param>
      public void SaveSettings(Dictionary<string, object> settings)
      {
         // no custom settings are required for this data source
      }

      /// <summary>
      /// This function is commanded when the user clicks the "Add -> Example Access Database" command  to add a new calendar
      /// data source. A form is displayed allowing the user to select which data source(s) they'd like to add. The example 
      /// database supports any number of different calendars.
      /// </summary>
      /// <param name="sources">The collection data sources, updated by the form if the user adds more calendars and clicks OK./</param>
      /// <returns>Returns DialogResult.OK if the user clicks the OK button on the displayed form.</returns>
      public DialogResult ShowAddCalendarForm(ICalendarCollection sources)
      {
         DialogResult result = DialogResult.Cancel;

         using (FormAddCalendar form = new FormAddCalendar(sources))
         {
            result = form.ShowDialog();
         }

         return result;
      }

      /// <summary>
      /// Gets the source name. This name is used in the PrintableCal calendar list and when comparing calendar data sources to
      /// determine which ones are implemented by this class. The name must be unique!
      /// </summary>
      public string SourceName
      {
         get { return SOURCE_NAME; }
      }

      /// <summary>
      /// Disposes references so garbage collection can eventually do its thing. Since this example class doesn't hold on to any
      /// references that need to be disposed, this function doesn't do anything. 
      /// </summary>
      public void Dispose()
      {
         // nothing to dispose
      }
   }
}
